import math
import numpy as np

def down_sample(msg_cache,msg_n):
    import math

    inc = len(msg_cache) / msg_n  # Calculate increment ratio
    inc_total = 0
    times = np.empty(msg_n)  # Initialize times dictionary
    n = 0
    
    for i in range(msg_n):
        index = math.floor(inc_total)
        if index < len(msg_cache):  # Check to avoid IndexError
            msg_downsampled = msg_cache[index]
            times[i] = msg_downsampled  # Assign to times array
            inc_total += inc
        else:
            break  # Exit loop if index exceeds msg_cache size

    return times


import glob
import os
import pandas as pd
import numpy as np
from pathlib import Path


def main():
    vedio_search_directory = 'S:/Sachuriga/Ephys_Vedio/CR_CA1/'
    folder_path = fr"S:/Sachuriga/Ephys_Recording/CR_CA1/65410/65410_2023-12-04_13-38-02_A/Record Node 102/"
    dlc =  load_positions(path,vedio_search_directory,folder_path,UD)

def load_positions(path,vedio_search_directory,folder_path,UD):

    ''' # Parameters:

        path: This parameter is not used in the function and can be removed.
        vedio_search_directory: This is the directory where the function will look for CSV files containing position data.
        folder_path: This is the directory where the function will look for .npy files containing timestamps and states.
        UD: This is a list or array-like object containing parameters used to construct the search pattern for the CSV files.
        Usage:

        Call the function with the appropriate parameters. For example:
        The function will print the search pattern it uses to find the CSV files.
        The function returns a numpy array containing the position data with timestamps inserted as the first column.
        Output:

        The function returns a numpy array where each row corresponds to a position sample. The first column of the array contains the timestamps, and the remaining columns contain the position data.
        Notes:

        The function assumes a specific directory structure and file naming convention. Make sure your files and directories match these expectations.
        The function only uses the first unique CSV file and the first unique .npy files it finds. If there are multiple matching files, only the first one is used.
        The function extracts specific columns from the CSV file. If your CSV file has a different structure, you may need to modify the column names in the code.
        The function assumes that state 3 in the states.npy file corresponds to the desired timestamps. If your states represent something different, you may need to modify the code.'''
    
    search_pattern = os.path.join(vedio_search_directory, f"*{UD[0]}*{UD[3]}{UD[1]}*600000_sk_filtered.csv")
    print(search_pattern)
    search_pattern1 = os.path.join(folder_path, '**/**/TTL/timestamps.npy')
    search_pattern3 = os.path.join(folder_path, '**/**/TTL/states.npy')
    #search_pattern2 = os.path.join(folder_path, '**/**/continuous/*/timestamps.npy')

    # Use glob to find files matching the pattern

    matching_files = glob.glob(search_pattern,recursive=True)
    matching_files = np.unique(matching_files)

    dlc_path=Path(matching_files[0])

    df = pd.read_csv(dlc_path)
    pos = df[['DLC_dlcrnetms5_CR_implant_DLCnetNov30shuffle3_600000',
                'DLC_dlcrnetms5_CR_implant_DLCnetNov30shuffle3_600000.1',
                'DLC_dlcrnetms5_CR_implant_DLCnetNov30shuffle3_600000.12',
                'DLC_dlcrnetms5_CR_implant_DLCnetNov30shuffle3_600000.13',
                'DLC_dlcrnetms5_CR_implant_DLCnetNov30shuffle3_600000.24',
                'DLC_dlcrnetms5_CR_implant_DLCnetNov30shuffle3_600000.25']]
    positions = np.float32(pos[3:].to_numpy())

    matching_files = glob.glob(search_pattern1,recursive=True)
    matching_files = np.unique(matching_files)
    v_time = np.load(matching_files[0])

    matching_files = glob.glob(search_pattern3,recursive=True)
    matching_files = np.unique(matching_files)
    v_state = np.load(matching_files[0])
    f_time = v_time[np.where(v_state==3)[0]]
    if f_time.shape[0] == 0:
        f_time = v_time[np.where(v_state==6)[0]]
        print('No timestamps found in the states file.')

    arr_with_new_col =  np.insert(positions , 0, f_time[:len(positions)], axis=1) # type: ignore
    return arr_with_new_col

def calc_head_direction(positions):
    """
    Calculate head direction.

    Calculates the head direction for each position sample pair. Direction
    is defined as east = 0 degrees, north = 90 degrees, west = 180 degrees,
    south = 270 degrees. Direction is set to NaN for missing samples.
    Position matrix contains information about snout and neck. Head
    direction is the counter-clockwise direction from back LED to the front.

    Parameters:
    positions (np.array): Animal's position data, Nx5. Position data should
                          contain timestamps (1 column), X/Y coordinates of
                          first LED (2 and 3 columns correspondingly), X/Y
                          coordinates of the second LED (4 and 5 columns
                          correspondingly).
                          it is assumed that positions[:, 1:2] correspond to
                          front LED, and positions[:, 3:4] to the back LED.
                          The resulting hd is the direction from back LED to
                          the front LED.

    Returns:
    np.array: Vector of head directions in degrees.
    """

    if positions.shape[1] < 5:
        raise ValueError('Position data should be 2D (type ''help calc_head_direction'' for details).')

    x1 = positions[:, 1]
    y1 = positions[:, 2]
    x2 = positions[:, 3]
    y2 = positions[:, 4]

    hd = np.remainder(np.arctan2(y2-y1, x2-x1) * 180 / np.pi + 180, 360)
    return hd

def moving_direction(pos, window_points=[1, 1], step=1):
    newPos = pos.copy()
    nBefore, nAfter = window_points

    if np.isnan(nBefore) or np.isnan(nAfter) or np.isnan(step):
        raise ValueError("Either 'windowPoints' or 'step' contains NaN values. This is not supported")

    num_samples = pos.shape[0]
    mdInd = np.arange(nBefore, num_samples - nAfter, step)

    kernel = np.concatenate([np.ones(nBefore)/nBefore, [0], -np.ones(nAfter)/nAfter])

    dropped_samples = np.setdiff1d(np.arange(num_samples), mdInd)
    newPos[dropped_samples, 1:] = np.nan

    if pos.shape[1] > 3:
        md = np.full((num_samples, 2), np.nan)
        md[mdInd, 0] = calc_direction(pos[:, 1], pos[:, 2], kernel, mdInd)
        md[mdInd, 1] = calc_direction(pos[:, 3], pos[:, 4], kernel, mdInd)
    else:
        md = np.full((num_samples, 1), np.nan)
        md[mdInd, 0] = calc_direction(pos[:, 1], pos[:, 2], kernel, mdInd)

    return md, newPos

def calc_direction(x, y, kernel, ind):
    X = np.convolve(x, kernel, 'same')[ind]
    Y = np.convolve(y, kernel, 'same')[ind]
    return np.mod(np.degrees(np.arctan2(Y, X)), 360)

if __name__== "__main__":
    main()


import sys
sys.path.append(r'Q:/sachuriga/Sachuriga_Python/quality_metrix')

from datetime import datetime
from zoneinfo import ZoneInfo
from pathlib import Path
from neuroconv.datainterfaces import PhySortingInterface
from neuroconv.datainterfaces import OpenEphysRecordingInterface
from neuroconv import ConverterPipe
from nwb4fp.postprocess.Get_positions import load_positions,calc_head_direction,moving_direction
from pynwb import NWBHDF5IO, NWBFile
from pynwb import NWBHDF5IO, NWBFile
from dateutil.tz import tzlocal
from nwb4fp.preprocess.down_sample_lfp import down_sample_lfp,add_lfp2nwb

from pynwb.behavior import (
    Position,
    SpatialSeries,
    CompassDirection
)
import numpy as np
def main():
    path = "S:\\Sachuriga/Ephys_Recording/CR_CA1/65409/65409_2023-12-08_16-39-36_A_phy_k_manual"
    sex = "F"
    ages = "P60"
    species = "Mus musculus"
    vedio_search_directory = "S:/Sachuriga/Ephys_Vedio/CR_CA1"
    path_to_save_nwbfile = "S:/Sachuriga/nwb"
    nwbPHYnOPHYS(path, sex, ages, species, vedio_search_directory, path_to_save_nwbfile)  # Added missing argument


def nwbPHYnOPHYS(path,sex,ages,species,vedio_search_directory,path_to_save_nwbfile):

    if path.endswith("phy_k_manual"):
        num2cal = int(41)
    elif path.endswith("phy_k"):
        num2cal = int(35)

    temp = path[0 - num2cal:]
    path1 = temp.split("/")

    file = path.split("_phy_")
    UD = path1[1].split("_")
    print(file[1])

    ECEPHY_DATA_PATH = file[0]
    stream_name = 'Record Node 102#OE_FPGA_Acquisition_Board-101.Rhythm Data'
    folder_path = fr"{ECEPHY_DATA_PATH}/Record Node 102"

    down_sample_lfp(fr"{path}",fr"{ECEPHY_DATA_PATH}")
    folder_path = Path(folder_path)

    try:
        interface_ophys = OpenEphysRecordingInterface(folder_path=folder_path,stream_name=stream_name)
    except AssertionError:
        try:
            folder_path = fr"{ECEPHY_DATA_PATH}/Record Node 102"
            stream_name = 'Record Node 102#OE_FPGA_Acquisition_Board-101.Rhythm Data'
            interface_ophys = OpenEphysRecordingInterface(folder_path=folder_path,stream_name=stream_name)
        except AssertionError:
            folder_path = fr"{ECEPHY_DATA_PATH}/Record Node 101"
            stream_name = 'Record Node 101#Acquisition_Board-100.Rhythm Data'
            interface_ophys = OpenEphysRecordingInterface(folder_path=folder_path,stream_name=stream_name)

    # Extract what metadata we can from the source files
    folder1_path = f"{path}"  # Change the folder_path to the location of the data in your system
    print(fr"Animal number is {UD[0]}")
    # if UD[0] == "65410":
    print(fr"Animal number is {UD[0]} replacing the spike times")
    #     sample_num = np.load(fr"{folder1_path}/spike_times.npy")
    #     timestemp = np.load(fr'{folder_path}\experiment1\recording1\continuous\OE_FPGA_Acquisition_Board-101.Rhythm Data/sample_numbers.npy')
    #     print(folder_path)
    #     time_spk = timestemp[sample_num]
    #     np.save(fr"{folder1_path}/spike_times.npy",time_spk)
        
    sample_num = np.load(fr"{folder1_path}/spike_times.npy")
    try:
        timestemp = np.load(fr'{folder_path}/continuous/OE_FPGA_Acquisition_Board-101.Rhythm Data/sample_numbers.npy')
    except FileNotFoundError:
        try:
            folder_path = fr"{ECEPHY_DATA_PATH}/Record Node 101"
            # S:\Sachuriga\Ephys_Recording\CR_CA1\65588\65588_2024-03-06_15-45-53_A\Record Node 101\experiment1\recording1\continuous\Acquisition_Board-100.Rhythm Data
            timestemp = np.load(fr'{folder_path}/experiment1/recording1/continuous/Acquisition_Board-100.Rhythm Data/sample_numbers.npy')
        except FileNotFoundError:
            try:
                folder_path = fr"{ECEPHY_DATA_PATH}/Record Node 102"
                timestemp = np.load(fr'{folder_path}/experiment1/recording1/continuous/OE_FPGA_Acquisition_Board-100.Rhythm Data/sample_numbers.npy')
            except FileNotFoundError:
                folder_path = fr"{ECEPHY_DATA_PATH}/Record Node 102"
                timestemp = np.load(fr'{folder_path}/experiment1/recording1/continuous/OE_FPGA_Acquisition_Board-101.Rhythm Data/sample_numbers.npy')
            
    print(folder_path)
    time_spk = timestemp[sample_num]

    np.save(fr"{folder1_path}/spike_times.npy",time_spk)
    
    interface_phy = PhySortingInterface(folder_path=folder1_path, verbose=False)
    # For data provenance we add the time zone information to the conversionSS

    converter = ConverterPipe(data_interfaces=[interface_ophys, interface_phy], verbose=False)
    # Extract what metadata we can from the source files
    metadata = converter.get_metadata()
    arr_with_new_col = load_positions(path,vedio_search_directory,folder_path,UD)
    # print(f"{arr_with_new_col.shape[1]} output the shape of the array")

    snout2neck = arr_with_new_col[:,[0,1,2,3,4]]
    neck2back4 = arr_with_new_col[:,[0,3,4,5,6]]

    # print(f"snout2neck: {snout2neck}")  
    # print(f"neck2back4: {neck2back4}")

    hd=calc_head_direction(snout2neck)
    bd=calc_head_direction(neck2back4)
    md,new_pos = moving_direction(arr_with_new_col )
    # print(f"position: {new_pos}")
    # print(f"moving Directions: {md.shape}")
    # print(f"Head Directions: {hd}")
    # print(f"Body Directions: {bd}")

    position_spatial_series = SpatialSeries(
        name="SpatialSeries",
        description="Position (x, y) in an open field.",
        data=arr_with_new_col[:,[1,2]],
        timestamps=arr_with_new_col[:,0],
        reference_frame="(0,0) is top left corner")
    
    hd_direction_spatial_series = SpatialSeries(name="SpatialSeries",
                                             description="View angle of the subject measured in radians.",
                                             data=hd,
                                             timestamps=snout2neck[:,0],
                                             reference_frame="straight ahead",
                                             unit="radians",)
    
    bd_direction_spatial_series = SpatialSeries(name="SpatialSeries",
                                             description="View angle of the subject measured in radians.",
                                             data=bd,
                                             timestamps=neck2back4[:,0],
                                             reference_frame="straight back",
                                             unit="radians",)

    md_direction_spatial_series = SpatialSeries(name="SpatialSeries",
                                             description="moving angle of the subject measured in radians.",
                                             data=md,
                                             timestamps=neck2back4[:,0],
                                             reference_frame="moving direction",
                                             unit="radians",)   

    nwbfile = NWBFile(
        session_description="Mouse exploring an open field",  # required
        identifier="sachuriga",  # required
        session_start_time=datetime(2020, 10, 31, 12, tzinfo=ZoneInfo("America/Los_Angeles")))  # required)
    behavior_module = nwbfile.create_processing_module(name="Behavioral data", 
                                                        description="position, head direction, and body direction of the mouse in an open field.")
    #ehavior_module = ProcessingModule(name="behavior", description="processed behavioral data")
    position = Position(spatial_series=position_spatial_series, name='Position in pixel')
    behavior_module.add(position)
    hd_direction = CompassDirection(spatial_series=hd_direction_spatial_series, name="Head(snout2neck)_Direction")
    bd_direction = CompassDirection(spatial_series=bd_direction_spatial_series, name="Body(neck2back4)_Direction")
    md_direction = CompassDirection(spatial_series=md_direction_spatial_series, name="Moving_Direction")

    behavior_module.add(hd_direction)
    behavior_module.add(bd_direction)
    behavior_module.add(md_direction)
    print(behavior_module)

    #position = Position(spatial_series=position_spatial_series)
    nwbfile_path = fr"{path_to_save_nwbfile}/{path1[1]}.nwb"
    io = NWBHDF5IO(nwbfile_path, mode="w")
    io.write(nwbfile)
    io.close()

    metadata = converter.get_metadata()
    metadata["NWBFile"]["experimenter"] = ["sachuriga,sachuriga"]
    metadata["Subject"] = dict(
        subject_id=UD[0],
        sex=sex,
        age=ages,
        species=species)
    metadata["general_session_id "] = UD[3]
    metadata["lab"] = "quattrocolo lab"
    metadata["institution"] = 'kavili institute for system neuroscience'
    metadata["session_description"] = f"{UD[3]}_room open-field CA1 recording"
    
    print(metadata)
    # For data provenance we add the time zone information to the conversionSS
    session_start_time = datetime.now(tzlocal())
    
    #metadata["NWBFile"].update(session_start_time=session_start_time)
    converter.run_conversion(nwbfile_path=nwbfile_path, metadata=metadata)
    print("completet!!!!adding conversion to nwb file")
    channel2selec = [3, 8, 24, 16, 10, 1, 31, 15, 31, 58, 53, 50, 52, 4, 63, 48, 45, 55]
    add_lfp2nwb(nwbfile_path,channel2selec,folder1_path)
    print("completet!!!!adding lfp to nwb file")
if __name__== "__main__":
    main()